class Scheduler {
  Scheduler();
  Scheduler.createAndExecute(void Function() callback) {
    _process(callback);
    _flushEvents();
  }
  final List<void Function()> _queue = [];
  bool _processingEvent = false;

  void dispose() {
    _queue.clear();
  }

  void schedule(void Function() task) {
    if (_processingEvent) {
      _queue.add(task);
      return;
    }

    if (_queue.isNotEmpty) {
      throw Exception(
          'Event queue should be empty when it is not processing events');
    }

    _process(task);
    _flushEvents();
  }

  void _flushEvents() {
    if (_queue.isEmpty) return;
    void Function()? nextCallback = _queue.removeAt(0);
    while (null != nextCallback) {
      _process(nextCallback);
      if (_queue.isEmpty) {
        nextCallback = null;
      } else {
        nextCallback = _queue.removeAt(0);
      }
    }
  }

  void _process(void Function() callback) {
    _processingEvent = true;
    try {
      callback();
    } catch (e, trace) {
      // there is no use to keep the future events
      // as the situation is not anymore the same
      _queue.clear();
      // Dart 2.16
      // Error.throwWithStackTrace(e, trace);
      rethrow;
    } finally {
      _processingEvent = false;
    }
  }
}
